package xyz.veronie.bgg.localdb;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;

import xyz.veronie.bgg.result.Thing;
import xyz.veronie.bgg.result.ThingMetaData;
import xyz.veronie.bgg.ui.helpers.Resources;

public class SqliteController {

	private static final SqliteController dbcontroller = new SqliteController();
    private static Connection connection;
    private String dbPath;

    static {
        try {
            Class.forName("org.sqlite.JDBC");
        } catch (ClassNotFoundException e) {
            System.err.println("Error while loading JDBC driver");
            e.printStackTrace();
        }
    }
    
    public Path getDbPath() {
    	return Paths.get(dbPath);
    }
    
    private SqliteController() {
    	dbPath = Resources.INSTANCE.getTmpDir() + File.separator + "bggtool.db";
    	initDBConnection();
    }
    
    public static SqliteController getInstance() {
        return dbcontroller;
    }
    
    private void initDBConnection() {
        try {
            if (connection != null)
                return;
            System.out.println("Creating Connection to Database...");
            connection = DriverManager.getConnection("jdbc:sqlite:" + dbPath);
            if (!connection.isClosed())
                System.out.println("...Connection established");
        } catch (SQLException e) {
            // throw new RuntimeException(e);
			System.out.println("Could not init local DB.");
        }

        Runtime.getRuntime().addShutdownHook(new Thread() {
            public void run() {
                try {
                    if (!connection.isClosed() && connection != null) {
                        connection.close();
                        if (connection.isClosed())
                            System.out.println("Connection to Database closed");
                    }
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        });
    }
    
    public void createSchema() {
    	try {
    		if(connection != null) {
		    	Statement stmt = connection.createStatement();
		        stmt.executeUpdate("CREATE TABLE IF NOT EXISTS Thing (ThingId INTEGER PRIMARY KEY, Name, ImgUrl, ThumbUrl, Comment, NumPlays);");
		        stmt.executeUpdate("CREATE TABLE IF NOT EXISTS ThingList (ListId INTEGER PRIMARY KEY, Name);");
		        stmt.executeUpdate("CREATE UNIQUE INDEX IF NOT EXISTS  idx1 ON ThingList(Name)");
		        stmt.executeUpdate("CREATE TABLE IF NOT EXISTS ThingListToThing (ListId, ThingId, FOREIGN KEY (ListId) REFERENCES ThingList(ListId), FOREIGN KEY (ThingId) REFERENCES Thing(ThingId));");
		        stmt.executeUpdate("CREATE UNIQUE INDEX IF NOT EXISTS idx2 ON ThingListToThing(ListId, ThingId)");
    		} else {
    			System.err.println("ERROR: Couldn't create Schema, connection is null.");
    		}
    	} catch (SQLException e) {
	        System.err.println("Couldn't create Schema");
	        e.printStackTrace();
    	}
    }

	public void openThingList(String name) {
		if(connection == null) {
			System.err.println("ERROR: Couldn't create Schema, connection is null.");
			return;			
		}

		try {
			Statement stmt = connection.createStatement();
			// insert new thing list if one with that name does not exist already
			stmt.executeQuery("INSERT INTO ThingList (name) VALUES ('" + name + "');");
		} catch (SQLException e) {
	        System.err.println("Couldn't add to thing list with handle " + name);
	        e.printStackTrace();
    	}
	}
	
	public void addToThingList(String name, List<Thing> things) {
		if(connection == null) {
			System.err.println("ERROR: Couldn't create Schema, connection is null.");
			return;			
		}

		try {
			
			Statement stmt = connection.createStatement();
			ResultSet res = stmt.executeQuery("SELECT ListId from ThingList where name = '" + name + "'");
			int listId = res.getInt(0);
			
			PreparedStatement thingStatement = connection
	                .prepareStatement("INSERT INTO Thing VALUES (?, ?, ?, ?, ?, ?) ON CONFLICT REPLACE;");
			PreparedStatement listToThingStatement = connection
					.prepareStatement("INSERT INTO ThingListToThing VALUES (?, ?)");
			
			for (Thing thing : things) {
				ThingMetaData metaData = thing.getMetaData();
				thingStatement.setInt(1, metaData.getId());
				thingStatement.setString(2, metaData.getName());
				thingStatement.setString(3, metaData.getImgURL());
				thingStatement.setString(4, metaData.getThumbURL());
				thingStatement.setString(5, metaData.getComment());
				thingStatement.setInt(6, metaData.getNumPlays());
				thingStatement.addBatch();
				// get generated id
				ResultSet keys = thingStatement.getGeneratedKeys();
				int thingId = keys.getInt(0);
				
				listToThingStatement.setInt(0, listId);
				listToThingStatement.setInt(1, thingId);
				listToThingStatement.addBatch();
			}
			
			connection.setAutoCommit(false);
            thingStatement.executeBatch();
            connection.setAutoCommit(true);
            
		} catch (SQLException e) {
	        System.err.println("Couldn't add to thing list with handle " + name);
	        e.printStackTrace();
    	}
	}


}
