package xyz.veronie.bgg.ui.filters;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.Creatable;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Layout;

import xyz.veronie.bgg.result.ResultConfigManager;
import xyz.veronie.bgg.types.EventConstants;
import xyz.veronie.bgg.types.FilterFlagState;
import xyz.veronie.bgg.types.Subtype;
import xyz.veronie.bgg.types.UserFlag;
import xyz.veronie.bgg.types.UserFlagEvent;
import xyz.veronie.bgg.ui.helpers.CommonControls;





/// These are the controls to retrieve thing IDs for a given BGG user
/// Filters can be set for the state of the thing in the user's collection
/// (i.e. owned, previously owned, wishlist, etc.)
@Creatable
public class BggUserSourceFilter {
	
	@Inject private IEventBroker eventBroker;
	@Inject private ResultConfigManager configManager;

	private Combo cbUserName;
	
	public void create(Composite parent, int style) {

		Composite mainCompo = new Composite(parent, SWT.FILL);
		mainCompo.setLayout(new GridLayout(5, false));
		mainCompo.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        
        Label lblUser = new Label(mainCompo, SWT.LEFT);
        lblUser.setText("User name: ");
        lblUser.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
//	     // Define field assists for the text widget
//	     // use "." and " " activate the content proposals
//	     char[] autoActivationCharacters = new char[] { '.', ' ' };
//	     KeyStroke keyStroke;
//	     try {
//	         keyStroke = KeyStroke.getInstance("Ctrl+Space");
//	         new ContentProposalAdapter(text, new TextContentAdapter(),
//	         new SimpleContentProposalProvider(new String[] { "ProposalOne", "ProposalTwo", "ProposalThree" }),
//	         keyStroke, autoActivationCharacters);
//	     } catch (ParseException e1) {
//	         e1.printStackTrace();
//	     }
        
        cbUserName = new Combo(mainCompo, SWT.DROP_DOWN);
        GridData gd = new GridData(SWT.FILL, SWT.FILL, false, false);
        gd.widthHint = 100;
        cbUserName.setLayoutData(gd);
        cbUserName.setText(configManager.getResultConfig().user);
        cbUserName.addSelectionListener(new SelectionAdapter() {
            public void widgetDefaultSelected(SelectionEvent e) {
                setUser(cbUserName.getText());
            }
        });
        cbUserName.addFocusListener(new FocusListener() {
			
			@Override
			public void focusLost(FocusEvent e) {
				setUser(cbUserName.getText());
			}
			
			@Override
			public void focusGained(FocusEvent e) {
				// nothing
			}
		});
        
        CommonControls.makeEmptySpace(mainCompo, 1);
        
//        Button btEditUsers = new Button(parent, SWT.PUSH);
//        btEditUsers.setText("edit users");
//        btEditUsers.setEnabled(false);
//        btEditUsers.setVisible(false);
//        btEditUsers.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
        // TODO: implement edit users

		// choose the thing sub-type
		Label lblSubtype = new Label(mainCompo, SWT.LEFT);
		lblSubtype.setText("Subtypes: ");
		lblSubtype.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false));
		
		ComboViewer cbSubtypes = new ComboViewer(mainCompo, SWT.READ_ONLY);
		cbSubtypes.getCombo().setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
		cbSubtypes.setContentProvider(ArrayContentProvider.getInstance());
		List<Subtype> subtypes = new ArrayList<Subtype>();
		for (Subtype st : Subtype.values()) { 
			subtypes.add(st);
		}
		cbSubtypes.setInput(subtypes);
		cbSubtypes.setSelection(new StructuredSelection(configManager.getResultConfig().subtype));
		cbSubtypes.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				eventBroker.send(EventConstants.TOPIC_SUBTYPE_CHANGED, selection.getFirstElement());
			}
		});
		
		
		
		Composite flagsComposite = new Composite(mainCompo, SWT.FILL);
		GridLayout layout = new GridLayout(4,false);
		GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
		gridData.horizontalSpan = 5;
		flagsComposite.setLayoutData(gridData);
		layout.horizontalSpacing = 15;
		flagsComposite.setLayout(layout);
		
        Label infoLabel = new Label(flagsComposite, SWT.LEFT);
        infoLabel.setText("Select which flags are used as filter. Filters follow 'AND' rule.");
        GridData gdInfo = new GridData(SWT.FILL, SWT.FILL, true, false);
        gdInfo.horizontalSpan = 4;
        infoLabel.setLayoutData(gdInfo);

        makeFilter(flagsComposite, UserFlag.OWN);
        makeFilter(flagsComposite, UserFlag.WTP);
        
        makeFilter(flagsComposite, UserFlag.PREVIOUSLY_OWNED);
        makeFilter(flagsComposite, UserFlag.WTB);
        
        makeFilter(flagsComposite, UserFlag.TRADE);
        makeFilter(flagsComposite, UserFlag.WISHLIST);
        
        makeFilter(flagsComposite, UserFlag.WANT);
        makeFilter(flagsComposite, UserFlag.PREORDERED);
        
        makeFilter(flagsComposite, UserFlag.RATED);
        makeFilter(flagsComposite, UserFlag.COMMENT);

        makeFilter(flagsComposite, UserFlag.PLAYED);

//        flagsComposite.pack();
//        flagsComposite.getParent().pack();
//        flagsComposite.layout();
        mainCompo.pack();
        mainCompo.layout(true, true);
	}
	
	
	/// Create a control for a filter. 
	// Filters are tri-state: include iff has flag, include iff it does not have the flag, 
	// don't care if it has that flag.
	// TODO: create a three-way toggle button instead. 
	private void makeFilter(Composite parent, final UserFlag key) {		
		
		ComboViewer filterCombo = new ComboViewer(parent, SWT.READ_ONLY);
		filterCombo.setContentProvider(ArrayContentProvider.getInstance());
		List<FilterFlagState> states = Arrays.asList(new FilterFlagState[] {
				FilterFlagState.IS, FilterFlagState.ISNOT, FilterFlagState.IGNORE
		});
		filterCombo.setInput(states);
		filterCombo.setSelection(new StructuredSelection(configManager.getResultConfig().userFlags.get(key)));
		filterCombo.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) event.getSelection();
				UserFlagEvent ufe = new UserFlagEvent();
		    	ufe.flag = key;
		    	ufe.state = (FilterFlagState)selection.getFirstElement();
				eventBroker.send(EventConstants.TOPIC_USERFLAG_CHANGED, ufe);
			}
		});
		
		Label filterLabel = new Label(parent, SWT.LEFT);
		filterLabel.setText(key.toString());
		filterLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
	}

	public void setUser(String user) {
		if(eventBroker != null) {
			eventBroker.post(EventConstants.TOPIC_USER_CHANGED, user);
		} else {
			System.out.println("setGeeklistId: eventBroker is null.");
		}
	}
	
}
